<?php
/*
 SPDX-FileCopyrightText: © 2013 Hewlett-Packard Development Company, L.P.

 SPDX-License-Identifier: GPL-2.0-only
*/

use Fossology\Lib\Dao\UploadDao;

/**
 * \file demomod.php
 * \brief browse an upload and display the demomod data (first bytes of the file)
 */

define("TITLE_UI_DEMOMOD", _("Demomod View"));

/**
 * @class ui_demomod
 * @brief browse an upload and display the demomod data (first bytes of the file)
 */
class ui_demomod extends FO_Plugin
{
  var $Name       = "demomod";        ///< Mod name
  var $Title      = TITLE_UI_DEMOMOD; ///< Page title
  var $Dependency = array("browse");  ///< Dependecy for plugin
  var $DBaccess   = PLUGIN_DB_READ;   ///< DB access required
  var $uploadtree_tablename;          ///< Upload tree table to use

  /**
   * \brief  Only used during installation.
   * \return 0 on success, non-zero on failure.
   */
  function Install()
  {
    global $PG_CONN;

    if (!$PG_CONN) {
      return(1);
    }

    return(0);
  } // Install()

  /**
   * \brief Customize submenus.
   */
  function RegisterMenus()
  {
    // For all other menus, permit coming back here.
    $URI = $this->Name . Traceback_parm_keep(array("upload","item"));
    $MenuName = "Demomod View";

    $Item = GetParm("item",PARM_INTEGER);
    $Upload = GetParm("upload",PARM_INTEGER);
    if (!empty($Item))
    {
      $text = _("Demomod data");
      menu_insert("Browse::$MenuName",100,$URI,$text);
      menu_insert("View::$MenuName",100,$URI,$text);
    }
  } // RegisterMenus()


  /**
   * \brief This is called before the plugin is used.
   *
   * It should assume that Install() was already run one time
   * (possibly years ago and not during this object's creation).
   *
   * \return true on success, false on failure.
   * A failed initialize is not used by the system.
   *
   * \note This function must NOT assume that other plugins are installed.
   */
  function Initialize()
  {
    if ($this->State != PLUGIN_STATE_INVALID) {  return(1);  // don't re-run
    }

    return($this->State == PLUGIN_STATE_VALID);
  } // Initialize()


  /**
   * \brief Display the demomod data
   *
   * \param $upload_pk
   * \param $uploadtree_pk
   */
  function ShowData($upload_pk, $uploadtree_pk)
  {
    global $PG_CONN;

    /* Check the demomod_ars table to see if we have any data */
    $sql = "select ars_pk from demomod_ars where upload_fk=$upload_pk and ars_success=true";
    $result = pg_query($PG_CONN, $sql);
    DBCheckResult($result, $sql, __FILE__, __LINE__);
    $rows = pg_num_rows($result);
    pg_free_result($result);

    if ($rows == 0) { return _("There is no demomod data for this upload.  Use Jobs > Schedule Agent.");
    }

    /* Get the scan result */
    /* First we need the pfile_pk */
    $sql = "select pfile_fk from $this->uploadtree_tablename where uploadtree_pk=$uploadtree_pk and upload_fk=$upload_pk";
    $result = pg_query($PG_CONN, $sql);
    DBCheckResult($result, $sql, __FILE__, __LINE__);
    $rows = pg_num_rows($result);
    if ($rows == 0) { return _("Internal consistency error. Failed: $sql");
    }
    $row = pg_fetch_assoc($result);
    $pfile_fk = $row['pfile_fk'];
    pg_free_result($result);

    /* Now we can get the scan result */
    $sql = "select firstbytes from demomod where pfile_fk=$pfile_fk";
    $result = pg_query($PG_CONN, $sql);
    DBCheckResult($result, $sql, __FILE__, __LINE__);
    $rows = pg_num_rows($result);
    if ($rows == 0) { return _("Internal consistency error. Failed: $sql");
    }
    $row = pg_fetch_assoc($result);
    $firstbytes = $row['firstbytes'];
    pg_free_result($result);

    $text = _("The first bytes of this file are: ");
    return ($text . $firstbytes);
  }


  /**
   * \brief This function returns the scheduler status.
   */
  function Output()
  {
    $uTime = microtime(true);
    if ($this->State != PLUGIN_STATE_READY) {
      return(0);
    }
    $V="";

    $Upload = GetParm("upload",PARM_INTEGER);
    /* @var $uploadDao UploadDao */
    $uploadDao = $GLOBALS['container']->get('dao.upload');
    if (!$uploadDao->isAccessible($Upload, Fossology\Lib\Auth\Auth::getGroupId()))
    {
      $text = _("Permission Denied");
      return "<h2>$text</h2>";
    }

    $Item = GetParm("item",PARM_INTEGER);
    $updcache = GetParm("updcache",PARM_INTEGER);

    /* Remove "updcache" from the GET args.
     * This way all the url's based on the input args won't be
     * polluted with updcache
     * Use Traceback_parm_keep to ensure that all parameters are in order */
    $CacheKey = "?mod=" . $this->Name . Traceback_parm_keep(array("upload","item","agent"));
    if ($updcache)
    {
      $_SERVER['REQUEST_URI'] = preg_replace("/&updcache=[0-9]*/","",$_SERVER['REQUEST_URI']);
      unset($_GET['updcache']);
      $V = ReportCachePurgeByKey($CacheKey);
    }
    else
    {
      $V = ReportCacheGet($CacheKey);
    }

    $this->uploadtree_tablename = GetUploadtreeTableName($Upload);

    if (empty($V) )  // no cache exists
    {
      switch($this->OutputType)
      {
        case "XML":
          break;
        case "HTML":
          $V .= "<font class='text'>\n";

          /************************/
          /* Show the folder path */
          /************************/
          $V .= Dir2Browse($this->Name,$Item,NULL,1,"Browse", -1, '', '', $this->uploadtree_tablename) . "<P />\n";

          if (!empty($Upload))
          {
            $Uri = preg_replace("/&item=([0-9]*)/","",Traceback());
            $V .= js_url();
            $V .= $this->ShowData($Upload, $Item);
          }
          $V .= "</font>\n";
          $V .= "<p>\n";
          break;
        case "Text":
          break;
        default:
      }

      $Cached = false;
    }
    else {
      $Cached = true;
    }

    if (!$this->OutputToStdout) {
      return($V);
    }
    print "$V";
    $Time = microtime(true) - $uTime;  // convert usecs to secs
    $text = _("Elapsed time: %.2f seconds");
    printf( "<small>$text</small>", $Time);

    if ($Cached)
    {
      $text = _("cached");
      $text1 = _("Update");
      echo " <i>$text</i>   <a href=\"$_SERVER[REQUEST_URI]&updcache=1\"> $text1 </a>";
    }
    else
    {
      /*  Cache Report if this took longer than 1/2 second*/
      if ($Time > 0.5) { ReportCachePut($CacheKey, $V);
      }
    }
    return;
  }

}

$NewPlugin = new ui_demomod;
$NewPlugin->Initialize();
